#set of functions, directly/indirectly involved, in oneChannelGUI generalToolMenu
#oneChannelGUI   filteringTable filtering a tab delimited file on the basis of a list of probesets
#oneChannelGUI geneExonLibs queries affymetrix web site to download exon and gene 1.0 ST WT library files
#oneChannelGUI aptFolder setting the folder were apt tools are located (save data in localdata.txt in etc)
#oneChannelGUI deleteLocalData delete data in localdata.txt resetting aptDir and libDirLocation

################################################################################
#this file contains a set of function that perfomr basic R function useful for handlying microarray data.
#this function filter a table on the basis of a vector of data present in an other file
"filteringTable" <- function(){
    Try(tkmessageBox(title="Table subsetting",message="Open the file to be filtered. This file should be tab delimited with an header."))
    Try(tempFileName <- tclvalue(tkgetOpenFile(filetypes="{{Text Files} {.txt}} {{All files} *}")))
    if(!nchar(tempFileName))
    {
       tkfocus(.affylmGUIglobals$ttMain)
       return()
    }
    Try(saving.file <- paste(tempFileName, ".filtered.by.",sep=""))
    Try(mySel<-read.table(tempFileName, sep="\t", header=T, as.is=TRUE, quote=""))
    Try(tkmessageBox(title="Table subsetting",message="Open the file to be used to filter the previous file.\nThis file should contain a single column with an header equal to \nthe one that should be used for filtering in the tab delimited file."))
    Try(tempFileName <- tclvalue(tkgetOpenFile(filetypes="{{Text Files} {.txt}} {{All files} *}")))
    if(!nchar(tempFileName))
    {
       tkfocus(.affylmGUIglobals$ttMain)
       return()
    }
    Try(tmp <- as.vector(unlist(strsplit(tempFileName , "/"))))
    Try(saving.file <- paste(saving.file, tmp[length(tmp)],sep=""))
    Try(myFilter<-read.table(tempFileName, sep="\t", header=T, as.is=TRUE, quote=""))
    Try(col.for.filter <- which(names(mySel)==names(myFilter)[1]))
    Try(mySel <- mySel[which(mySel[,col.for.filter]%in%myFilter[,1]),])
    Try(tkmessageBox(title="Table subsetting",message="Saving the filtered table."))
	  Try(FileName <- tclvalue(tkgetSaveFile(initialfile=saving.file, filetypes="{{Tab-Delimited Text Files} {.txt .xls}} {{All files} *}")))
    Try(if(!nchar(FileName))
		    	return())
		Try(write.table(mySel,file=FileName,quote=FALSE,col.names=NA,sep="\t"))
   
}
################################################################################
#save affy libraries in allylibs dir as new dir in oneChannelPath
 "geneExonLibs" <- function(){
      Try(tkmessageBox(title="Gene Exon 1.0 ST libraries",message="This routine installs, in a user defined folder,\nall library files needed for analysis of Gene/Exon Affymetrix arrays."))
      Try(myWD <- getwd())
      #Try(SetWD())
      Try(libDirLocation <- get("libDirLocation",envir=affylmGUIenvironment))
      if(libDirLocation != ""){
             Try(mbVal <- tkmessageBox(title="Gene Exon 1.0 ST libraries",
				    											message="There already a folder defined for Affymetrix libraries.\nDo you want to change it?",
						    									icon="question",type="yesno",default="no"
								    						 )#end of tkmessageBox
		        )#end of Try(mbVal..
    		    if(tclvalue(mbVal)=="no"){
			            Try(tkfocus(.affylmGUIglobals$ttMain))
			            Try(setwd(myWD))
			            return()
		       } else{
                    Try(.downloadLibs())
                    return()
           } 
      }
   Try(.downloadLibs())
   Try(setwd(myWD))
}

".downloadLibs" <- function(){
                     Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="watch"))
                     Try(libDirLocation  <- paste(.path.package("oneChannelGUI", quiet = FALSE), "/affylibs", sep=""))
                     Try(dir.create(libDirLocation, showWarnings = TRUE, recursive = FALSE))
                     Try(setwd(libDirLocation))
                     Try(assign("libDirLocation", libDirLocation, affylmGUIenvironment))#directory where affy library files are located
                     Try(cat("\nBegin downloads of library files.....\n"))
                     Try(cat("Be patient!\n"))
                     Try(download.file("http://www.bioinformatica.unito.it/downloads/HuEx-1_0-st-v2.r2.zip", "HuEx-1_0-st-v2.r2.zip", mode="wb"))
                     Try(download.file("http://www.bioinformatica.unito.it/downloads/HuGene-1_0-st-v1.r3.analysis_libraryfile.zip","HuGene-1_0-st-v1.r3.analysis_libraryfile.zip", mode="wb"))
                     Try(download.file("http://www.bioinformatica.unito.it/downloads/MoEx-1_0-st-v1.r2.zip", "MoEx-1_0-st-v1.r2.zip", mode="wb"))
                     Try(download.file("http://www.bioinformatica.unito.it/downloads/MoGene-1_0-st-v1.r3.analysis-lib-files.zip","MoGene-1_0-st-v1.r3.analysis-lib-files.zip", mode="wb"))
                     Try(download.file("http://www.bioinformatica.unito.it/downloads/RaEx-1_0-st-v1.r2.zip","RaEx-1_0-st-v1.r2.zip", mode="wb"))
                     Try(download.file("http://www.bioinformatica.unito.it/downloads/RaGene-1_0-st-v1.r3.analysis-lib-files.zip","RaGene-1_0-st-v1.r3.analysis-lib-files.zip", mode="wb"))

               #      Try(download.file("http://www.affymetrix.com/Auth/support/downloads/library_files/HuEx-1_0-st-v2.r2.zip", "HuEx-1_0-st-v2.r2.zip", method="wget"))
               #      Try(download.file("http://www.affymetrix.com/Auth/support/downloads/library_files/MoEx-1_0-st-v1_libraryfile.zip", "MoEx-1_0-st-v1_libraryfile.zip"))
               #      Try(download.file("http://www.affymetrix.com/Auth/support/downloads/library_files/RaEx-1_0-st-v1_libraryfile.zip","RaEx-1_0-st-v1_libraryfile.zip"))
               #      Try(download.file("http://www.affymetrix.com/Auth/support/downloads/library_files/HuGene-1_0-st-v1.r3.analysis_libraryfile.zip","HuGene-1_0-st-v1.r3.analysis_libraryfile.zip"))
                     Try(cat("\nEnd downloads of 6 library files.....\n"))
                     Try(cat("\nBegin unzip library files.....\n"))
                     if(.Platform$OS.type=="windows"){
                         Try(unzipped1 <- zip.unpack("HuGene-1_0-st-v1.r3.analysis_libraryfile.zip", getwd()))
                         Try(if(!unzipped1) cat("\nHuGene correctly unzipped\n"))
                         Try(unzipped2 <- zip.unpack("HuEx-1_0-st-v2.r2.zip", getwd()))
                         Try(if(!unzipped2) cat("\nHuEx correctly unzipped\n"))
                         Try(unzipped3 <- zip.unpack("MoEx-1_0-st-v1.r2.zip", getwd()))
                         Try(if(!unzipped3) cat("\nMoEx correctly unzipped\n"))
                         Try(unzipped4 <- zip.unpack("MoGene-1_0-st-v1.r3.analysis-lib-files.zip", getwd()))
                         Try(if(!unzipped4) cat("\nMoGene correctly unzipped\n"))
                         Try(unzipped5 <- zip.unpack("RaEx-1_0-st-v1.r2.zip", getwd()))
                         Try(if(!unzipped5) cat("\nRaEx correctly unzipped\n"))
                         Try(unzipped6 <- zip.unpack("RaGene-1_0-st-v1.r3.analysis-lib-files.zip", getwd()))
                         Try(if(!unzipped6) cat("\nRaGene correctly unzipped\n"))
                     } else{
                       Try(system(paste("unzip HuGene-1_0-st-v1.r3.analysis_libraryfile.zip", sep="")))
                       Try(system(paste("unzip HuEx-1_0-st-v2.r2.zip")))
                       Try(system(paste("unzip MoEx-1_0-st-v1.r2.zip")))
                       Try(system(paste("unzip MoGene-1_0-st-v1.r3.analysis-lib-files.zip")))
                       Try(system(paste("unzip RaEx-1_0-st-v1.r2.zip")))
                       Try(system(paste("unzip RaGene-1_0-st-v1.r3.analysis-lib-files.zip")))
                     }
                     Try(cat("\nEnd unzip library files.....\n"))
                     Try(cat("\nLibrary files are ready to be used in ", getwd(),"\n"))
                     #save the info to be recovered in a new session
                     Try(etcDir  <- paste(.path.package("oneChannelGUI", quiet = FALSE), "/etc", sep=""))
                     Try(localdata <- paste(etcDir, "/localData.txt", sep=""))
                     Try(myinfo <- read.table(localdata, sep="\t", header=F, as.is=T))
                     Try(myinfo[which(myinfo[,1]=="libDirLocation"),2] <- libDirLocation)
                     Try(write.table(myinfo, localdata, sep="\t", row.names=F, col.names=F, quote=F))
                     Try(tkconfigure(.affylmGUIglobals$ttMain,cursor="arrow"))
}
#######################################################################################################
#define the folder where apt tools are locate
"aptFolder" <- function(){
      Try(tkmessageBox(title="APT tools folder",message="Please select the folder where APT tools are located."))
      Try(myWD <- getwd())
      #Try(SetWD())
      Try(aptDir <- get("aptDir",envir=affylmGUIenvironment))
      if(is.na(aptDir)){aptDir <- ""}
      if(aptDir == ""){
            Try(SetED())
            Try(aptDir <- getwd())
            if(length(grep("apt-probeset-summarize", dir(paste(aptDir, "/bin", sep="")))) == 0){
                      Try(tkmessageBox(title="APT tools dir ",message="You have not correctly defined the main folder where APT tools are located!",icon="error"))
                      assign("aptDir.Available",FALSE,affylmGUIenvironment)
                      assign("aptDir", "" ,affylmGUIenvironment)
                      Try(setwd(myWD))
                      Try(return())
            } else {
                       Try(localdata  <- paste(.path.package("oneChannelGUI", quiet = FALSE), "/etc/localData.txt", sep=""))
                       Try(myinfo <- read.table(localdata, sep="\t", header=F, as.is=T))
                       Try(myinfo[which(myinfo[,1]=="aptDir"),2] <- aptDir)
                       Try(write.table(myinfo, localdata, sep="\t", row.names=F, col.names=F, quote=F))
                       assign("aptDir.Available",TRUE,affylmGUIenvironment)
                       assign("aptDir", aptDir ,affylmGUIenvironment)
                       Try(setwd(myWD))
                       Try(return())
            }
      } else{
                      Try(mbVal <- tkmessageBox(title="APT tools folder",
				    											message="There already a folder defined for APT tools.\nDo you want to change it?",
						    									icon="question",type="yesno",default="no"
								    						 )#end of tkmessageBox
		                  )#end of Try(mbVal..
    		              if(tclvalue(mbVal)=="no"){
			                        Try(tkfocus(.affylmGUIglobals$ttMain))
			                        Try(setwd(myWD))
			                        return()
		                  } 
                      if(tclvalue(mbVal)=="yes"){
                              Try(SetED())
                              Try(aptDir <- getwd())
                              if(length(grep("apt-probeset-summarize", dir(paste(aptDir, "/bin", sep="")))) == 0){
                                  Try(tkmessageBox(title="APT tools dir ",message="You have not correctly defined the main folder where APT tools are located!",icon="error"))
                                  assign("aptDir.Available",FALSE,affylmGUIenvironment)
                                  assign("aptDir", "" ,affylmGUIenvironment)
                                  Try(setwd(myWD))
                                  Try(return())
                              } else {
                                 Try(localdata  <- paste(.path.package("oneChannelGUI", quiet = FALSE), "/etc/localData.txt", sep=""))
                                 Try(myinfo <- read.table(localdata, sep="\t", header=F, as.is=T))
                                 Try(myinfo[which(myinfo[,1]=="aptDir"),2] <- aptDir)
                                 Try(write.table(myinfo, localdata, sep="\t", row.names=F, col.names=F, quote=F))
                                 assign("aptDir.Available",TRUE,affylmGUIenvironment)
                                 assign("aptDir", aptDir ,affylmGUIenvironment)
                                 Try(setwd(myWD))
                                 Try(return())
                             }
                      }      
           }
}
###########################################################################################
"deleteLocalData" <- function(){
          Try(mbVal <- tkmessageBox(title="Local setting",
				    											message="Do you want to the delete the local setting for Gene Exon arrays analysis?",
						    									icon="question",type="yesno",default="no"
								    						 )#end of tkmessageBox
		        )#end of Try(mbVal..
    		    if(tclvalue(mbVal)=="no"){
			            Try(tkfocus(.affylmGUIglobals$ttMain))
			            return()
		       } else{
                    Try(etcDir  <- paste(.path.package("oneChannelGUI", quiet = FALSE), "/etc", sep=""))
                    Try(localdata <- paste(etcDir, "/localData.txt", sep=""))
                    Try(myinfo <- read.table(localdata, sep="\t", header=F, as.is=T))
                    Try(myinfo[which(myinfo[,1]=="libDirLocation"),2] <- "")
                    Try(myinfo[which(myinfo[,1]=="aptDir"),2] <- "")
                    Try(write.table(myinfo, localdata, sep="\t", row.names=F, col.names= F, quote=F))
                    Try(tkmessageBox(title="Local settings",message="Local setting were reset."))
                    return()
           } 

}
